/*

------------------------------------------------------------------------------

A license is hereby granted to reproduce this software source code and
to create executable versions from this source code for personal,
non-commercial use.  The copyright notice included with the software
must be maintained in all copies produced.

THIS PROGRAM IS PROVIDED "AS IS". THE AUTHOR PROVIDES NO WARRANTIES
WHATSOEVER, EXPRESSED OR IMPLIED, INCLUDING WARRANTIES OF
MERCHANTABILITY, TITLE, OR FITNESS FOR ANY PARTICULAR PURPOSE.  THE
AUTHOR DOES NOT WARRANT THAT USE OF THIS PROGRAM DOES NOT INFRINGE THE
INTELLECTUAL PROPERTY RIGHTS OF ANY THIRD PARTY IN ANY COUNTRY.

Copyright (c) 2004-2005, John Conover, All Rights Reserved.

Comments and/or bug reports should be addressed to:

    john@email.johncon.com (John Conover)

------------------------------------------------------------------------------

mboxstats.c is for counting the number of messages in a Unix mbox. The
mbox file is specified on the command line, (or defaults to stdin,)
and statistics of the messages are printed to stdout.

$Revision: 0.0 $
$Date: 2005/01/18 19:36:00 $
$Id: mboxstats.c,v 0.0 2005/01/18 19:36:00 john Exp $
$Log: mboxstats.c,v $
Revision 0.0  2005/01/18 19:36:00  john
Initial version


*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <unistd.h>

static char rcsid[] = "$Id: mboxstats.c,v 0.0 2005/01/18 19:36:00 john Exp $"; /* program version */
static char copyright[] = "Copyright (c) 2004-2005, John Conover, All Rights Reserved"; /* the copyright banner */

#define BUFLEN BUFSIZ /* i/o buffer size */
#define MAX_ALPHABET_SIZE 256 /* maximum size of alphabet, characters numbered 0 .. 255 */

#ifdef __STDC__

static const char *help_message[] = /* help message index array */

#else

static char *help_message[] = /* help message index array */

#endif

{
    "\n",
    "Count the messages in a Unix mbox\n",
    "Usage: mboxstats [-t] [-f] [-s] [-v] [filename]\n",
    "    -t, don't output the \"To: \" lines\n",
    "    -f, don't output the \"From: \" lines\n",
    "    -s, don't output \"Subject: \" lines\n",
    "    -v, print the program's version information\n",
    "    filename, input filename\n"
};

#ifdef __STDC__

static const char *error_message[] = /* error message index array */

#else

static char *error_message[] = /* error message index array */

#endif

{
    "No error\n",
    "Error in program argument(s)\n",
    "Error opening file\n",
    "Error closing file\n",
};

#define NOERROR 0 /* error values, one for each index in the error message array */
#define EARGS 1
#define EOPEN 2
#define ECLOSE 3

#ifdef __STDC__

static void print_message (int retval); /* print any error messages */

#else

static void print_message (); /* print any error messages */

#endif

#ifdef __STDC__

int main (int argc, char *argv[])

#else

int main (argc, argv)
int argc;
char *argv[];

#endif

{
    char buffer[BUFLEN], /* i/o buffer */
         cbuffer[BUFLEN], /* copy of i/o buffer */
         to[BUFLEN], /* contains the To: address */
         from[BUFLEN], /* contains the From: address */
         subject[BUFLEN], /* contains the Subject: record */
         uppercase[MAX_ALPHABET_SIZE], /* capitalized character set */
         *ref, /* character reference */
         c; /* character */

    int count = 0, /* input file record counter */
        size = 0, /* input file character counter */
        i, /* character counter */
        total_count = 0, /* total records */
        total_size = 0, /* total bytes */
        messages = 0, /* count of messages */
        pt = 1, /* print the To: records, 0 = no, 1 = yes */
        pf = 1, /* print the From: records, 0 = no, 1 = yes */
        ps = 1, /* print the Subject: records, 0 = no, 1 = yes */
        retval = NOERROR; /* return value, assume no error */

    FILE *infile = stdin; /* reference to input file */

    while ((c = (char) getopt (argc, argv, "tfsv")) != EOF) /* for each command line switch */
    {

        switch (c) /* which switch? */
        {

            case 't': /* print To: records? */

                pt = 0; /* set the print the To: records, 0 = no, 1 = yes */
                break;

            case 'f': /* print From: records? */

                pf = 0; /* set the print the From: records, 0 = no, 1 = yes */
                break;

            case 's': /* print Subject: records? */

                ps = 0; /* set the print the Subject: records, 0 = no, 1 = yes */
                break;

            case 'v':

                (void) printf ("%s\n", rcsid); /* print the version */
                (void) printf ("%s\n", copyright); /* print the copyright */
                optind = argc; /* force argument error */
                retval = EARGS; /* assume not enough arguments */

            default: /* illegal switch? */

                optind = argc; /* force argument error */
                retval = EARGS; /* assume not enough arguments */
                break;
        }

    }

    if (retval == NOERROR) /* enough arguments? */
    {

        for (i = 0; i < MAX_ALPHABET_SIZE; i++) /* for each character in the character set */
        {
            uppercase[i] = (char) toupper (i); /* no, convert the character to uppercase */
        }

        retval = EOPEN; /* assume error opening file */

        if ((infile = argc <= optind ? stdin : fopen (argv[optind], "r")) != (FILE *) 0) /* yes, open the input file */
        {
            retval = NOERROR; /* assume no error */
            count = 0; /* initialize the input file record counter */
            size = 0; /* initialize the input file character counter */
            to[0] = '\0'; /* null all header buffers */
            from[0] = '\0'; /* null all header buffers */
            subject[0] = '\0'; /* null all header buffers */
            total_count = 0; /* initialize the total records */
            total_size = 0; /* initialize the total bytes */
            messages = 0; /* initialize the count of messages */

            while (fgets (buffer, BUFLEN, infile) != (char *) 0) /* read the records from the input file */
            {
                (void) strcpy (cbuffer, buffer); /* make a copy of the record */

                if (strchr (cbuffer, (int) '\n')) /* look for an EOL, to prevent buffer overflow-if buffer overflow, skip the record */
                {
                    ref = cbuffer; /* for each character in the record */
                    i = 0; /* initialize the character counter */

                    while (*ref != '\0' && i < BUFLEN)
                    {
                        *ref = uppercase[(size_t) *ref]; /* transiterate the character to upper case */
                        ref ++; /* next character */
                        i ++; /* one more count */
                    }

                    if (strncmp (cbuffer, "TO: ", 4) == 0) /* From: line? */
                    {

                        if (to[0] == '\0') /* first To: in message? */
                        {
                            (void) strcpy (to, buffer); /* yes, save it */
                        }

                    }

                    if (strncmp (cbuffer, "FROM: ", 6) == 0) /* From: line? */
                    {

                        if (from[0] == '\0') /* first From: in message? */
                        {
                            (void) strcpy (from, buffer); /* yes, save it */
                        }

                    }

                    if (strncmp (cbuffer, "SUBJECT: ", 9) == 0) /* Subject: line? */
                    {

                        if (subject[0] == '\0') /* first Subject: in message? */
                        {
                            (void) strcpy (subject, buffer); /* yes, save it */
                        }

                    }

                    if (strncmp (cbuffer, "FROM ", 5) == 0) /* begining of message? */
                    {
                        messages ++; /* one more message */

                        if (count != 0) /* if first record, skip it */
                        {

                            if (to[0] != '\0') /* any To: field? */
                            {

                                if (pt == 1) /* print the To: records, 0 = no, 1 = yes? */
                                {
                                    (void) printf ("%s", to); /* yes, print it */
                                }

                            }

                            if (from[0] != '\0') /* any From: field? */
                            {

                                if (pf == 1) /* print the From: records, 0 = no, 1 = yes? */
                                {
                                    (void) printf ("%s", from);  /* yes, print it */
                                }

                            }

                            if (subject[0] != '\0') /* any Subject field? */
                            {

                                if (ps == 1) /* print the Subject: records, 0 = no, 1 = yes? */
                                {
                                    (void) printf ("%s", subject);  /* yes, print it */
                                }

                            }

                            (void) printf ("\t%d\t%d\n", size, count); /* yes, print the stats */
                            total_count = total_count + count; /* add the total records */
                            total_size = total_size + size; /* add the total bytes */
                            count = 0; /* initialize the input file record counter */
                            size = 0; /* initialize the input file character counter */
                            to[0] = '\0'; /* null all header buffers */
                            from[0] = '\0'; /* null all header buffers */
                            subject[0] = '\0'; /* null all header buffers */
                        }

                    }

                    count ++; /* one more record */
                    size = size + i; /* length of line */
                }

            }

            if (to[0] != '\0') /* any To: field? */
            {

                if (pt == 1) /* print the To: records, 0 = no, 1 = yes? */
                {
                    (void) printf ("%s", to); /* yes, print it */
                }

            }

            if (from[0] != '\0') /* any From: field? */
            {

                if (pf == 1) /* print the From: records, 0 = no, 1 = yes? */
                {
                    (void) printf ("%s", from);  /* yes, print it */
                }

            }

            if (subject[0] != '\0') /* any Subject field? */
            {

                if (ps == 1) /* print the Subject: records, 0 = no, 1 = yes? */
                {
                    (void) printf ("%s", subject);  /* yes, print it */
                }

            }

            (void) printf ("\t%d\t%d\n", size, count); /* yes, print the stats */
            total_count = total_count + count; /* add the total records */
            total_size = total_size + size; /* add the total bytes */
            (void) printf ("%d messages, %d bytes, %d lines, total\n", messages, total_size, total_count); /* print the final stats */

            if (argc > optind) /* using stdin as input? */
            {

                if (fclose (infile) == EOF) /* no, close the input file */
                {
                    retval = ECLOSE; /* error closing file */
                }

            }

        }

    }

    print_message (retval); /* print any error messages */
    exit (retval); /* exit with the error value */

#ifdef LINT

    return (0); /* for lint formalities */

#endif

}

/*

Print any error messages.

static void print_message (int retval);

I) Data structures:

    A) The help_message array is an array of character stings, one per
       line to be printed for help.

    B) The error_message array is an array of character strings, one
       line per error; the error_message array is implicitly addressed
       by the value integer retval, which specifies the error to be
       printed.

II) Function execution:

    A) Depending on the value of the integer retval:

        1) If retval is zero, print nothing-normal/successful program
           exit.

        2) Else if retval is unity, print help.

        3) Else retval is not zero or unity, it is an error code, print
           the corresponding error message.

Returns nothing.

*/

#ifdef __STDC__

static void print_message (int retval)

#else

static void print_message (retval)
int retval;

#endif

{
    size_t help_ctr; /* help_message line counter */

    switch (retval) /* which return value */
    {

        case 0: /* program ended without errors, print nothing */

            break;

        case 1: /* program ended with a request for help, or argument error, print help */

            for (help_ctr = 0; help_ctr < (sizeof (help_message) / sizeof (char *)); help_ctr ++) /* for each line of help */
            {
                (void) printf ("%s", help_message[help_ctr]); /* print the line */
            }

            break;

        default: /* an error that was not a request for help, print the error */

            (void) fprintf (stderr, "%s", error_message[retval]);
            break;

    }

}
